@@ -53,14 +53,14 @@ static long rrd_fetch_dbi_long(dbi_result *result,int idx) {
       } else if (attr & DBI_INTEGER_SIZE4) { value=dbi_result_get_int_idx(result,idx);
       } else if (attr & DBI_INTEGER_SIZE8) { value=dbi_result_get_longlong_idx(result,idx);
       } else {                               value=DNAN;
-        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %i for type INTEGER\n",time(NULL),idx,attr ); }
+        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %u for type INTEGER\n",time(NULL),idx,attr ); }
       }
       break;
     case DBI_TYPE_DECIMAL:
       if        (attr & DBI_DECIMAL_SIZE4) { value=floor(dbi_result_get_float_idx(result,idx));
       } else if (attr & DBI_DECIMAL_SIZE8) { value=floor(dbi_result_get_double_idx(result,idx));
       } else {                               value=DNAN;
-        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %i for type DECIMAL\n",time(NULL),idx,attr ); }
+        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %u for type DECIMAL\n",time(NULL),idx,attr ); }
       }
       break;
     case DBI_TYPE_BINARY:
@@ -82,7 +82,7 @@ static long rrd_fetch_dbi_long(dbi_result *result,int idx) {
        value=dbi_result_get_datetime_idx(result,idx);
        break;
     default:
-      if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported type: %i with attribute %i\n",time(NULL),idx,type,attr ); }
+      if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported type: %u with attribute %u\n",time(NULL),idx,type,attr ); }
       value=DNAN;
       break;
   }
@@ -116,14 +116,14 @@ static double rrd_fetch_dbi_double(dbi_result *result,int idx) {
       } else if (attr & DBI_INTEGER_SIZE4) { value=dbi_result_get_int_idx(result,idx);
       } else if (attr & DBI_INTEGER_SIZE8) { value=dbi_result_get_longlong_idx(result,idx);
       } else {                               value=DNAN;
-        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %i for type INTEGER\n",time(NULL),idx,attr ); }
+        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %u for type INTEGER\n",time(NULL),idx,attr ); }
       }
       break;
     case DBI_TYPE_DECIMAL:
       if        (attr & DBI_DECIMAL_SIZE4) { value=dbi_result_get_float_idx(result,idx);
       } else if (attr & DBI_DECIMAL_SIZE8) { value=dbi_result_get_double_idx(result,idx);
       } else {                               value=DNAN;
-        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %i for type DECIMAL\n",time(NULL),idx,attr ); }
+        if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported attribute flags %u for type DECIMAL\n",time(NULL),idx,attr ); }
       }
       break;
     case DBI_TYPE_BINARY:
@@ -145,7 +145,7 @@ static double rrd_fetch_dbi_double(dbi_result *result,int idx) {
        value=dbi_result_get_datetime_idx(result,idx);
        break;
     default:
-      if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported type: %i with attribute %i\n",time(NULL),idx,type,attr ); }
+      if (getenv("RRDDEBUGSQL")) { fprintf(stderr,"RRDDEBUGSQL: %li: column %i unsupported type: %u with attribute %u\n",time(NULL),idx,type,attr ); }
       value=DNAN;
       break;
   }
@@ -313,7 +313,7 @@ static char* _find_next_separator_twice(char*start,char separator) {
       /* and return the pointer past the current one*/
       return (found+2);
     }
-    /* find next occurence */
+    /* find next occurrence */
     found=strchr(found+1,separator);
   }
   /* not found, so return NULL */
@@ -642,7 +642,7 @@ rrd_fetch_fn_libdbi(
       r_value=DNAN;
       /* check for timestamp delta to be within an acceptable range */
       if ((d_timestamp>0)&&(d_timestamp<2*derive)) {
-	/* only handle positive delta - avoid wrap-arrounds/counter resets showing up as spikes */
+	/* only handle positive delta - avoid wrap-arounds/counter resets showing up as spikes */
 	if (d_value>0) {
 	  /* and normalize to per second */
 	  r_value=d_value/d_timestamp;
