@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
+ * RRDtool 1.7.2 Copyright by Tobi Oetiker, 1997-2019
  *****************************************************************************
  * rrd_info  Get Information about the configuration of an RRD
  *****************************************************************************/
@@ -8,6 +8,9 @@
 #include "rrd_rpncalc.h"
 #include "rrd_client.h"
 #include <stdarg.h>
+#ifdef _MSC_VER
+#include "asprintf.h"   /* for vasprintf() here */
+#endif
 
 /* allocate memory for string */
 char     *sprintf_alloc(
