@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
+ * RRDtool 1.7.2 Copyright by Tobi Oetiker, 1997-2019
  *****************************************************************************
  * rrd_modify  Structurally modify an RRD file
  *      (c) 2014 by Peter Stamfest and Tobi Oetiker
@@ -1426,7 +1426,7 @@ int handle_modify(const rrd_t *in, const char *outfilename,
 	    }
 	    rraopcnt++;
 	} else {
-	    rrd_set_error("unparseable argument: %s", argv[i]);
+	    rrd_set_error("unparsable argument: %s", argv[i]);
 	    rc = -1;
 	    goto done;
 	}
